@@ -3,15 +3,10 @@ from __future__ import absolute_import, division, print_function
 from pymongo import MongoClient
 from confluent_kafka import Producer
 import json
-import copy
-try:
-    from urllib.parse import quote_plus  # Python 3
-except ImportError:
-    from urllib import quote_plus  # Python 2
+from urllib.parse import quote_plus
 
 from vivarium.library.dict_utils import (
-    merge_dicts, value_in_embedded_dict, get_path_list_from_dict, \
-    get_value_from_path, make_path_dict)
+    value_in_embedded_dict, make_path_dict)
 
 HISTORY_INDEXES = [
     'time',
@@ -80,7 +75,6 @@ def get_emitter(config):
 
 def configure_emitter(config, processes, topology):
     emitter_config = config.get('emitter', {})
-    emitter_config['keys'] = get_emitter_keys(processes, topology)
     emitter_config['experiment_id'] = config.get('experiment_id')
     emitter_config['simulation_id'] = config.get('simulation_id')
     return get_emitter(emitter_config)
@@ -130,7 +124,7 @@ class Emitter(object):
         print(data)
 
     def get_data(self):
-        return []
+        return {}
 
     def get_path_timeseries(self):
         return path_timeseries_from_data(self.get_data())
@@ -150,7 +144,7 @@ class NullEmitter(Emitter):
 class TimeSeriesEmitter(Emitter):
 
     def __init__(self, config):
-        keys = config.get('keys', {})
+        super().__init__(config)
         self.saved_data = {}
 
     def emit(self, data):
@@ -177,7 +171,7 @@ class KafkaEmitter(Emitter):
     >>> emitter = KafkaEmitter(config)
     '''
     def __init__(self, config):
-        self.config = config
+        super().__init__(config)
         self.producer = Producer({
             'bootstrap.servers': self.config['host']})
 
@@ -209,7 +203,7 @@ class DatabaseEmitter(Emitter):
     default_host = 'localhost:27017'
 
     def __init__(self, config):
-        self.config = config
+        super().__init__(config)
         self.experiment_id = config.get('experiment_id')
 
         # create singleton instance of mongo client
